@@ -32,6 +32,14 @@ import types
 
 # Project Imports
 
+def _fullClassName(obj):
+    if isinstance(obj, type):
+        cls = obj
+    else:
+        cls = obj.__class__
+
+    return '%s.%s' % (cls.__module__, cls.__name__)
+
 class Event(object):
     """
     The action that caused a state transition, it has a type and any other
@@ -50,6 +58,9 @@ class Event(object):
         """
         self.type = etype
 
+        for arg, name in kwargs.iteritems():
+            setattr(self, arg, name)
+
 def _eventFromFrame(frame, name):
     """
     Automatically creates an event from the given inspect.Frame object and name.
@@ -76,11 +87,11 @@ def _eventFromFrame(frame, name):
 def declareEventType(name):
     """
     Defines an event type in a manner which will avoid collisions
-    
+
     It defines it in the following format: <file>:<line> <EVENT>
-    
-    @note All spaces in the string will be replace with '_' 
-    
+
+    @note All spaces in the string will be replace with '_'
+
     @rtype : str
     @return: The new event type
     """
@@ -154,19 +165,19 @@ class State(object):
         Called when the state is exited, loopbacks don't count
         """
         pass
-    
+
     def publish(self, eventType, event):
         """
         Publish an event, with the owning Machine object as publisher
-        
+
         @warning: Only valid when the object is created by a Machine object
         """
-        raise 
-        
+        raise
+
 class End(State):
     """
     State machine which demarks a valid end point for a state machine
-    
+
     Ensure that any "dead ends" (states with no out transitions) are actually
     intended to be that way.
     """
@@ -181,7 +192,7 @@ class Branch(object):
     """
     def __init__(self, state, branchingEvent = None):
         """
-        @type state: ram.ai.state.State
+        @type state: state.State
         @param state: The state to branch to
 
         @type branchingEvent: Event
@@ -193,34 +204,34 @@ class Branch(object):
 class Machine(object):
     """
     An event based finite state machine.
-    
+
     This machine works with graph of statepy.State classes.  This graph
     represents a state machine.  There can only be one current state at a time.
-    When events are injected into the state machine the currents states 
+    When events are injected into the state machine the currents states
     transition table determines which state to advance to next.
-    
+
     @type _root: statepy.State
     @ivar _root: The first state of the Machine
-    
+
     @type _currentState: statepy.State
     @ivar _currentState: The current state which is processing events
-    
+
     @type _started: boolean
     @ivar _started: The Machine will not process events unless started
 
     @type _started: boolean
     @ivar _started: True when the Machine is complete (ie. currentState = None)
-        
+
     @type _previousEvent: Event
     @ivar _previousEvent: The last event injected into the state machine
 
     @todo statevars, branches
     """
-    
+
     STATE_ENTERED = declareEventType('STATE_ENTERED')
     STATE_EXITED = declareEventType('STATE_EXITED')
     COMPLETE = declareEventType('COMPLETE')
-    
+
     def __init__(self, statevars = None):
         """
         The constructor for the Machine class.
@@ -228,7 +239,7 @@ class Machine(object):
         @type  statevars: dict
         @param statevars: A dictionary of the object variables given to states
         """
-        
+
         if statevars is None:
             statevars = {}
 
@@ -241,17 +252,17 @@ class Machine(object):
         self._statevars = {}
         self._startStatevars = {}
         self._branches = {}
-        
+
         # Load up the arguments
         self._statevars = statevars
-        
+
     def currentState(self):
         return self._currentState
 
-    def start(self, startState, statevars = None):
+    def start(self, startState, statevars = None, startEvent = None):
         """
         Starts or branches the state machine with the given state
-        
+
         If the given state is really a branch, it will branch to that state
         instead.
 
@@ -260,6 +271,9 @@ class Machine(object):
 
         @type  statevars: dict
         @param statevars: An additional dictionary of variables for the State
+
+        @type  stateEvent: Event
+        @param stateEvent: Optional event to given when entering a state
         """
 
         # Remove the previous startStatevars from our list of variables
@@ -275,12 +289,12 @@ class Machine(object):
             if len(intersection) != 0:
                 msg = "ERROR: statevars already contains: %s" % interection
                 raise statepy.StatePyException(msg)
-            
+
             self._startStatevars = statevars
 
             # Merge the statevars
             self._statevars.update(statevars)
-        
+
         if Branch == type(startState):
             # Determine if we are branching
             branching = True
@@ -290,7 +304,7 @@ class Machine(object):
             self._started = True
             self._complete = False
 
-            self._enterState(startState)
+            self._enterState(startState, startEvent)
 
     def stop(self):
         """
@@ -299,17 +313,17 @@ class Machine(object):
         """
         if self._currentState is not None:
             self._exitState()
-        
+
         self._started = False
         self._previousEvent = Event
         self._root = None
         self._currentState = None
         self._complete = False
-        
+
         for branch in self._branches.itervalues():
             branch.stop()
         self._branches = {}
-        
+
     def stopBranch(self, stateType):
         """
         Stops just the desired branch, and its current type
@@ -320,20 +334,20 @@ class Machine(object):
     def injectEvent(self, rawEvent, _sendToBranches = False):
         """
         Sends an event into the state machine
-        
-        If currents states transition table has an entry for events of this 
+
+        If currents states transition table has an entry for events of this
         type this will cause a transition
-        
+
         @type  event: Event or str
-        @param event: A new event for the state machine to process 
-        
+        @param event: A new event for the state machine to process
+
         @type _sendToBranches: bool
-        @param _sendToBranches: Use only for testing, injects events into 
+        @param _sendToBranches: Use only for testing, injects events into
                                 branched state machines
         """
         # If the state we just entered transitions on same kind of event that
         # caused the transition, we can be notified again with the same event!
-        # This check here prevents that event from causing an unwanted 
+        # This check here prevents that event from causing an unwanted
         # transition.
         if rawEvent == self._previousEvent:
             return
@@ -343,10 +357,10 @@ class Machine(object):
             event = rawEvent
         else:
             event = Event(rawEvent)
-        
+
         if not self._started:
             raise Exception("Machine must be started")
-        
+
         transitionTable = self._currentState.transitions()
         nextState = transitionTable.get(event.type, None)
         if nextState is not None:
@@ -355,23 +369,23 @@ class Machine(object):
             if Branch == type(nextState):
                 branching = True
                 nextState = nextState.state
-                
+
             # Detemine if we are in a loopback
             loopback = False
             if nextState == type(self._currentState):
                 loopback = True
-                
-            # For loops backs or branches we don't reenter, or exit from our 
+
+            # For loops backs or branches we don't reenter, or exit from our
             # state, just call the transition function
             leaveState = False
             if (not branching) and (not loopback):
                 leaveState = True
-            
+
             # We are leaving the current state
             currentState = self._currentState
             if leaveState:
                 self._exitState()
-            
+
             # Call the function for the transitions
             transFunc = self._getTransitionFunc(event.type, currentState)
             if transFunc is not None:
@@ -380,93 +394,125 @@ class Machine(object):
             # Notify that we are entering the next state
             if (not loopback) and (not branching):
                 # Create an instance of the next state's class
-                self._enterState(nextState)
+                self._enterState(nextState, event)
             elif branching:
                 self._branchToState(nextState, branchingEvent = event)
-                
+
         # Record previous event
         self._previousEvent = event
-        
+
         if _sendToBranches:
             for branch in self._branches.itervalues():
-                branch.injectEvent(event)
+                branch.injectEvent(event, _sendToBranches = _sendToBranches)
 
     @property
     def complete(self):
         """Returns true when """
         return self._complete
 
-    def _enterState(self, newStateClass):
+    def _enterState(self, newStateClass, event=None):
         """
         Does all the house keeping when entering a new state
         """
-        
+
         # CONFIG LOOKUP USE TO HAPPEN HERE
-        
+
         # Create state instance from class, make sure to pass all subsystems
         # along as well
         newState = newStateClass(**self._statevars)
-        
+
         # Subscribe to every event of the desired type
         # <EVENT SUBSCRIBTION USE TO HAPPEN HERE>
         transitionTable = newState.transitions()
-        
+
         # Actual enter the state and record it as our new current state
         self._currentState = newState
-        self._currentState.enter()
-        
+
+        if event is None:
+            self._currentState.enter()
+        else:
+            # This is a little bit of hack, but we see if the enter function
+            # can actually take an additional argument, and if it can we pass
+            # in the event.
+            spec = inspect.getargspec(self._currentState.enter)
+
+            if len(spec.args) > 1 or spec.varargs is not None or \
+                   spec.keywords is not None:
+                self._currentState.enter(event)
+            else:
+                self._currentState.enter()
+
         # Notify everyone we just entered the state
-        #fullClassName = '%s.%s' % (self._currentState.__class__.__module__, 
-        #                           self._currentState.__class__.__name__)
+        #fullClassName = _fullClassName(self._currentState)
         # NOTIFY STATE BEING ENTERED
-        
+
         # If we are in a state with no way out, exit the state and mark ourself
         # complete
         if 0 == len(transitionTable):
-            self._exitState()
+            # Mark complete before running the exit function, so any transitions
+            # triggered during the exit find the machine complete.
             self._complete = True
+
+            self._exitState()
+
+            # Mark it not started so that we get a clear error message when there is
+            # no current events
+            self._started = False
+
             # NOTIFY MACHINE BEING COMPLETE
-        
+
     def _exitState(self):
         """
         Does all the house keeping for when you are exiting an old state
         """
         self._currentState.exit()
-                
+
         # NOTIFY STATE BEING EXIT
-        #fullClassName = '%s.%s' % (self._currentState.__class__.__module__, 
-        #                           self._currentState.__class__.__name__)
-        
+        #fullClassName = _fullClassName(self._currentState)
+
         self._currentState = None
 
     def _branchToState(self, nextState, branchingEvent = None):
+
+        # Remove any branches that are complete
+        done = []
+        for key, branchMachine in self._branches.iteritems():
+            if branchMachine.complete:
+                done.append(key)
+
+        for k in done:
+            del self._branches[k]
+
+        # We can't branch to the same state twice
         if self._branches.has_key(nextState):
-            raise Exception("Already branched to this state")
-        
+            args = _fullClassName(nextState)
+            raise Exception("Already branched to this state: %s" % args)
+
         # Create new state machine
         branchedMachine = Machine(self._statevars)
 
+        # Store new state machine
+        self._branches[nextState] = branchedMachine
+
         # Start it up with the proper state
-        branchedMachine.start(nextState)
+        branchedMachine.start(nextState, startEvent=branchingEvent)
 
         # Set the previous state to avoid unwanted transitions caused by
         # the event that led us hear, triggering a transition in the newly
         # created state machine
         branchedMachine._previousEvent = branchingEvent
 
-        # Store new state machine
-        self._branches[nextState] = branchedMachine
 
     def _getTransitionFunc(self, etype, obj):
         """
         Determines which funtion during a transistaion between states
-        
+
         This uses the event type of the event which caused the transition to
         determine which member funtion of the self._currentState to call.
         """
         # Trim etype of namespace stuff
         etype = etype.split(' ')[-1]
-        
+
         # Grab all member functions
         members = inspect.getmembers(obj, inspect.ismethod)
 
@@ -487,29 +533,29 @@ class Machine(object):
         """
         Write the graph of the state machine starting at the given state to
         the fileobj.
-    
+
         @type  fileobj: a file like object
         @param fileobj: The object to write the result graph to (ie:
                         fileobject.write(graphtext))
-       
-        @type  startState: ram.ai.state.State
+
+        @type  startState: state.State
         @param startState: The state to start the graph at
-        
+
         @type  ordered: boolean
         @param ordered: Whether or not to alphabetize the states
         """
         fileobj.write("digraph aistate {\n")
         stateTransitionList = []
         traversedStates = []
-        
+
         Machine._traverse(startState, stateTransitionList, traversedStates,
                           noLoops)
-        
+
         # Sort list for determinism
         if ordered:
             stateTransitionList.sort()
 
-        # Output Labels in Simple format        
+        # Output Labels in Simple format
         traversedStates.sort(key = Machine._dottedName)
         for state in traversedStates:
             fullName = Machine._dottedName(state)
@@ -525,7 +571,7 @@ class Machine(object):
             fileobj.write(item + "\n")
         fileobj.write("}")
         fileobj.flush() # Push data to file
-        
+
     @staticmethod
     def _traverse(currentState,stateList,traversedList,noLoops=False):
         if 0 == len(currentState.transitions()):
@@ -534,13 +580,13 @@ class Machine(object):
         else:
             for aiEvent,aiState in currentState.transitions().iteritems():
                 eventName = str(aiEvent).split(' ')[-1]
-                
+
                 # Style is determine whether or not we are branching
                 style = "solid"
                 if type(aiState) is Branch:
                     style = "dotted"
                     aiState = aiState.state
-                
+
                 # Determine state names
                 startName = Machine._dottedName(currentState)
                 endName = Machine._dottedName(aiState)
